@@ -26,8 +26,8 @@ import("core.base.global")
 import("core.project.project")
 import("core.platform.platform")
 import("devel.debugger")
-import("private.async.runjobs")
-import("private.action.run.make_runenvs")
+import("async.runjobs")
+import("private.action.run.runenvs")
 import("private.service.remote_build.action", {alias = "remote_build_action"})
 
 -- run target
@@ -44,23 +44,18 @@ function _do_run_target(target)
     -- get the absolute target file path
     local targetfile = path.absolute(target:targetfile())
 
-    -- add run environments
-    local addrunenvs, setrunenvs = make_runenvs(target)
-    for name, values in pairs(addrunenvs) do
-        os.addenv(name, table.unpack(table.wrap(values)))
-    end
-    for name, value in pairs(setrunenvs) do
-        os.setenv(name, table.unpack(table.wrap(value)))
-    end
+    -- build run environments
+    local addenvs, setenvs = runenvs.make(target)
 
     -- get run arguments
     local args = table.wrap(option.get("arguments") or target:get("runargs"))
 
     -- debugging?
     if option.get("debug") then
-        debugger.run(targetfile, args, {curdir = rundir})
+        debugger.run(targetfile, args, {curdir = rundir, addenvs = addenvs, setenvs = setenvs})
     else
-        os.execv(targetfile, args, {curdir = rundir, detach = option.get("detach")})
+        local envs = runenvs.join(addenvs, setenvs)
+        os.execv(targetfile, args, {curdir = rundir, detach = option.get("detach"), envs = envs})
     end
 end
 
@@ -213,6 +208,14 @@ function main()
     -- load config first
     config.load()
 
+    -- Automatically build before running
+    if project.policy("run.autobuild") then
+        -- we need clear the previous config and reload it
+        -- to avoid trigger recheck configs
+        config.clear()
+        task.run("build")
+    end
+
     -- load targets
     project.load_targets()
 
